#include "lcd_ili9488.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define LCD_ILI9488_DUMMY 0

static uint8_t line_image [(LCD_ILI9488_SCREEN_WIDTH * BYTES_PER_PIXEL)] = { 0 };
static uint8_t cmd_2c = LCD_ILI9488_WRITE_RAM;

static uint8_t col_start_stop[4];
static uint8_t row_start_stop[4];

static uint8_t LCD_CMD_1[ 3 ] = { 0x00, 0x62, 0x3B }; //Display Inversion Control
static uint8_t LCD_CMD_3[ 4 ] = { 0x00, 0x00, 0x01, 0x3F}; // set portrait memory layout
static uint8_t LCD_CMD_4[ 4 ] = { 0x00, 0x00, 0x01, 0xEF}; // set portrait memory layout
static uint8_t LCD_CMD_5[ 4 ] = { 0x00, 0x00, 0x01, 0xEF}; // set landscape memory layout
static uint8_t LCD_CMD_6[ 4 ] = { 0x00, 0x00, 0x01, 0x3F}; // set landscape memory layout

// NOTE: if you add more colors please remember to add them in the lcd_ili9488.h file
struct RGB_COLOR colors[] = {
    { 0x00, 0x00, 0x00 }, // BLACK
    { 0xff, 0x00, 0x00 }, // RED
    { 0x00, 0xff, 0x00 }, // GREEN
    { 0x00, 0x00, 0xff }, // BLUE
    { 0xff, 0xff, 0xff }, // WHITE
    { 0x00, 0xff, 0xff }, // CYAN
    { 0xff, 0xff, 0x00 }, // YELLOW
    { 0xdc, 0x10, 0x10}   // DEEP_RED
};

// ---------------------------------------------- PRIVATE FUNCTION DECLARATIONS 

static void box_area 
( 
    lcd_ili9488_t *ctx, 
    uint16_t start_col, 
    uint16_t start_row, 
    uint16_t end_col, 
    uint16_t end_row, 
    uint16_t color 
);


static void draw_area_rgb
( 
    lcd_ili9488_t *ctx, 
    uint16_t start_col, 
    uint16_t start_row, 
    uint16_t end_col, 
    uint16_t end_row, 
    const uint8_t *img,
    uint16_t rgb_bits
);

static void pixel( lcd_ili9488_t *ctx, uint16_t col, uint16_t row, uint16_t color );

static void character( lcd_ili9488_t *ctx, uint16_t ch );

static void _circle 
( 
    lcd_ili9488_t *ctx, 
    uint16_t x0, 
    uint16_t y0, 
    uint16_t r, 
    uint16_t color 
);


// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void lcd_ili9488_cfg_setup ( lcd_ili9488_cfg_t *cfg )
{
    cfg->spi_speed = 20000000; // 20MHZ
    cfg->spi_mode = SPI_MASTER_MODE_0;
    cfg->cs_polarity = SPI_MASTER_CHIP_SELECT_POLARITY_ACTIVE_LOW;
}

LCD_ILI9488_RETVAL lcd_ili9488_init ( lcd_ili9488_t *ctx, lcd_ili9488_cfg_t *cfg )
{
    spi_master_config_t spi_cfg;

    spi_master_configure_default( &spi_cfg );
    spi_cfg.speed     = cfg->spi_speed;
    spi_cfg.sck       = cfg->sck;
    spi_cfg.miso      = cfg->miso;
    spi_cfg.mosi      = cfg->mosi;
    spi_cfg.default_write_data = LCD_ILI9488_DUMMY;

    digital_out_init( &ctx->cs, cfg->cs );
    ctx->chip_select = cfg->cs;

    if (  spi_master_open( &ctx->spi, &spi_cfg ) == SPI_MASTER_ERROR )
    {
        return LCD_ILI9488_INIT_ERROR;
    }

    spi_master_set_default_write_data( &ctx->spi, LCD_ILI9488_DUMMY );
    spi_master_set_speed( &ctx->spi, cfg->spi_speed );
    spi_master_set_mode( &ctx->spi, cfg->spi_mode );
    spi_master_set_chip_select_polarity( cfg->cs_polarity );

    // Output pins 
    
    digital_out_init( &ctx->rw, cfg->rw );
    digital_out_init( &ctx->rst, cfg->rst );
    digital_out_init( &ctx->dc, cfg->dc );
    digital_out_init( &ctx->en, cfg->en );

    digital_out_high( &ctx->dc );
    spi_master_deselect_device( ctx->chip_select );   

    return LCD_ILI9488_OK;
}

void lcd_ili9488_enable ( lcd_ili9488_t *ctx, uint8_t state )
{
    digital_out_write( &ctx->en, state );
}

void lcd_ili9488_reset( lcd_ili9488_t *ctx )
{
    digital_out_high( &ctx->rst );
    Delay_ms(50);
    digital_out_low( &ctx->rst );
    Delay_ms(50);
    digital_out_high( &ctx->rst );
    Delay_ms(50);
}

void lcd_ili9488_more_arg_commands ( lcd_ili9488_t *ctx, uint8_t command, uint8_t *args, uint16_t args_len )
{
    uint16_t cnt;

    spi_master_select_device( ctx->chip_select );
    digital_out_low( &ctx->dc );
    spi_master_write( &ctx->spi, &command, 1 );
    digital_out_high( &ctx->dc );
    
    for ( cnt = 0; cnt < args_len; cnt++ )
    {
        spi_master_write( &ctx->spi, &args[cnt], 1 );
    }
    spi_master_deselect_device( ctx->chip_select );  
}

void lcd_ili9488_one_arg_commands ( lcd_ili9488_t *ctx, uint8_t command, uint8_t args )
{
    spi_master_select_device( ctx->chip_select );
    digital_out_low( &ctx->dc );
    spi_master_write( &ctx->spi, &command, 1 );
    digital_out_high( &ctx->dc );
    spi_master_write( &ctx->spi, &args, 1 );
    spi_master_deselect_device( ctx->chip_select );  
}

void lcd_ili9488_default_cfg ( lcd_ili9488_t *ctx )
{
    lcd_ili9488_enable( ctx, 1 ); 
    lcd_ili9488_reset( ctx );

    lcd_ili9488_more_arg_commands( ctx, LCD_ILI9488_SW_RESET,  0,  0 ); // SWRESET Software reset
    Delay_ms( 100 ); //wait for wake up ~ 100ms
    lcd_ili9488_more_arg_commands( ctx, LCD_ILI9488_SLEEP_OUT,  0,  0 ); // Sleep OUT
    Delay_ms( 20 ); //wait for wake up ~ 20ms

    lcd_ili9488_one_arg_commands( ctx, LCD_ILI9488_DISP_INV_CTL, 0x00 ); // Display Inversion Control, Disable Z-inversion
    lcd_ili9488_more_arg_commands( ctx, LCD_ILI9488_DISP_FUN_CTL ,  LCD_CMD_1, 3 ); // Display Function Control

    lcd_ili9488_one_arg_commands( ctx, LCD_ILI94880_INTER_PIX_FORMAT, 0x66 ); // Interface Pixel Format = 18bit,
                                                     // CPU Interface = 18bit
                                                     // color format RRRRRRxx GGGGGGxx BBBBBBxx 262K color
                                                     // this is a limitation of the SPI buss
    #ifndef LANDSCAPE
    lcd_ili9488_one_arg_commands( ctx, LCD_ILI9488_MEMORY_MODE, 0x88 ); // Memory Access Control Portrait, RGB
    lcd_ili9488_more_arg_commands( ctx, LCD_ILI9488_SET_COL_ADDRESS ,  LCD_CMD_3, 4 ); // set portrait memory layout
    lcd_ili9488_more_arg_commands( ctx, LCD_ILI9488_SET_ROW_ADDRESS ,  LCD_CMD_4, 4 ); // set portrait memory layout
    #else
    lcd_ili9488_one_arg_commands( ctx, LCD_ILI9488_MEMORY_MODE, 0x28 ); // Memory Access Control Landscape mode, RGB
    lcd_ili9488_more_arg_commands( ctx, LCD_ILI9488_SET_COL_ADDRESS ,  LCD_CMD_5, 4 ); // set landscape memory layout
    lcd_ili9488_more_arg_commands( ctx, LCD_ILI9488_SET_ROW_ADDRESS ,  LCD_CMD_6, 4 ); // set landscape memory layout
    #endif

    lcd_ili9488_more_arg_commands( ctx, LCD_ILI9488_DISPLAY_ON,  0,  0 ); //Display ON

}

void lcd_ili9488_fill_screen ( lcd_ili9488_t *ctx, uint16_t color )
{
// set start and ending address for the whole screen landscape or portrait
    #ifndef LANDSCAPE
    lcd_ili9488_one_arg_commands( ctx, LCD_ILI9488_MEMORY_MODE, 0x88 ); // Memory Access Control Portrait, RGB org
    lcd_ili9488_more_arg_commands( ctx, LCD_ILI9488_SET_COL_ADDRESS ,  LCD_CMD_3, 4 ); // set landscape memory layout
    lcd_ili9488_more_arg_commands( ctx, LCD_ILI9488_SET_ROW_ADDRESS ,  LCD_CMD_4, 4 ); // set landscape memory layout
    #else
    lcd_ili9488_one_arg_commands( ctx, LCD_ILI9488_MEMORY_MODE, 0x28 ); // Memory Access Control Landscape mode, RGB
    lcd_ili9488_more_arg_commands( ctx, LCD_ILI9488_SET_COL_ADDRESS ,  LCD_CMD_5, 4 ); // set landscape memory layout
    lcd_ili9488_more_arg_commands( ctx, LCD_ILI9488_SET_ROW_ADDRESS ,  LCD_CMD_6, 4 ); // set landscape memory layout
    #endif

// load the line buffer with color
    for (uint16_t x = 0; x < (LCD_ILI9488_SCREEN_WIDTH * BYTES_PER_PIXEL); x = x + BYTES_PER_PIXEL)
        {
        line_image[x] = colors[color].r; // fill the array with a color (red)
        line_image[x+1] = colors[color].g; // fill the array with a color (green)
        line_image[x+2] = colors[color].b; // fill the array with a color (blue)
        }
// send the lines to the display ram
    spi_master_select_device( ctx->chip_select );
    digital_out_low( &ctx->dc );
    spi_master_write( &ctx->spi, &cmd_2c, 1 );
    digital_out_high( &ctx->dc );
    
    for (uint16_t y_cnt = 0; y_cnt < LCD_ILI9488_SCREEN_HEIGHT; y_cnt++)
        {
            spi_master_write( &ctx->spi, &line_image[0], (LCD_ILI9488_SCREEN_WIDTH * BYTES_PER_PIXEL) );
        }
    spi_master_deselect_device( ctx->chip_select );  
}

void lcd_ili9488_rectangle
( 
    lcd_ili9488_t *ctx, 
    uint16_t col_off, 
    uint16_t row_off, 
    uint16_t col_end, 
    uint16_t row_end, 
    uint16_t color 
)
{
    box_area( ctx, col_off, row_off, col_end, row_end, color );
}

void lcd_ili9488_image( lcd_ili9488_t *ctx, const uint8_t* img, uint16_t col_off, uint16_t row_off, uint16_t rgb_bits)
{
    const uint8_t *ptr = img;
    uint16_t x1;
    uint16_t y1;

    x1 = (((img[2])) | ((img[3]) << 8)) -1;
    y1 = (((img[4])) | ((img[5]) << 8)) -1;

    draw_area_rgb( ctx, col_off, row_off, col_off + y1, row_off + x1, ptr, rgb_bits );
}

void lcd_ili9488_text( lcd_ili9488_t *ctx, uint8_t *text, uint16_t x, uint16_t y )
{
    uint8_t *ptr = text;

    if ( ( x >= LCD_ILI9488_SCREEN_WIDTH ) || ( y >= LCD_ILI9488_SCREEN_HEIGHT ) )
    {
        return;
    }
    
    ctx->x_cord = x;
    ctx->y_cord = y;

    while( *ptr )
    {
        character( ctx, *ptr++ );
    }
}

void lcd_ili9488_set_font( lcd_ili9488_t *ctx, const uint8_t *font_s, uint16_t color )
{
    ctx->font_obj               = font_s;
    ctx->font_first_char    = font_s[2] + (font_s[3] << 8);
    ctx->font_last_char     = font_s[4] + (font_s[5] << 8);
    ctx->font_height        = font_s[6];
    ctx->font_color         = color;
}

void lcd_ili9488_circle
(
    lcd_ili9488_t *ctx,
    uint16_t x0,
    uint16_t y0,
    uint16_t r,
    uint16_t color,
    uint16_t width
)
{
    uint8_t i = 0;
    if (width < 1) width = 1; // trap a 0 and set 1 to draw something
    if (width > r) width = r; // trap line width wider than radius set to the same
    i = width;
    while (i > 0)
    {
        _circle( ctx, x0, y0, (r-i), color);
        i--;
    }
}

void lcd_ili9488_fill_circle 
( 
    lcd_ili9488_t *ctx, 
    uint16_t x0, 
    uint16_t y0, 
    uint16_t r, 
    uint16_t color 
)
{
    int16_t x;
    int16_t y;
    int16_t e2;
    int16_t err;

    x = -r;
    y = 0;
    err = 2 -2*r;

    while (x <= 0) {
        lcd_ili9488_line(ctx, x0 -x, y0 -y, x0 -x, y0 +y, color);
        lcd_ili9488_line(ctx, x0 +x, y0 -y, x0 +x, y0 +y, color);
        e2 = err;
        if (e2 <= y) {
            y++;
            err = err + y*2 +1;
            if (-x == y && e2 <= x) {
                e2 = 0;
            }
        }
        if ( e2 > x) {
            x++;
            err = err + x*2 +1;
        }
    }
}

void lcd_ili9488_line
(
    lcd_ili9488_t *ctx,
    uint16_t x1,
    uint16_t y1,
    uint16_t x2,
    uint16_t y2,
    uint16_t color
)
{
    int dx = abs(x2 - x1);
    int dy = abs(y2 - y1);
    int sx = x1 < x2 ? 1 : -1;
    int sy = y1 < y2 ? 1 : -1;
    int err = dx - dy;

    while (1) {
        pixel( ctx, x1, y1, color );
        if (x1 == x2 && y1 == y2) break; // Line is complete
        int e2 = 2 * err;
        if (e2 > -dy) {
            err -= dy;
            x1 += sx;
        }
        if (e2 < dx) {
            err += dx;
            y1 += sy;
        }
    }    
}

// ----------------------------------------------- PRIVATE FUNCTION DEFINITIONS

static void _circle 
( 
    lcd_ili9488_t *ctx, 
    uint16_t x0, 
    uint16_t y0, 
    uint16_t r, 
    uint16_t color 
)
{
    int16_t x;
    int16_t y;
    int16_t e2;
    int16_t err;

    x = -r;
    y = 0;
    err = 2 -2*r;

    while (x <= 0) {
        pixel( ctx, x0 - x, y0 +y, color );
        pixel( ctx, x0 + x, y0 +y, color );
        pixel( ctx, x0 + x, y0 -y, color );
        pixel( ctx, x0 - x, y0 -y, color );
        e2 = err;
        if (e2 <= y) {
            y++;
            err = err + y*2 +1;
            if (-x == y && e2 <= x) {
                e2 = 0;
            }
        }
        if ( e2 > x) {
            x++;
            err = err + x*2 +1;
        }
    }
}

static void box_area 
( 
    lcd_ili9488_t *ctx, 
    uint16_t start_col, 
    uint16_t start_row, 
    uint16_t end_col, 
    uint16_t end_row, 
    uint16_t color 
)
{
    uint8_t   cmd       = LCD_ILI9488_WRITE_RAM;
    uint32_t  cnt       = ( end_col - start_col ) * ( end_row - start_row );
    uint8_t   clr[ 3 ]  = { 0 };

    if( ( start_col > LCD_ILI9488_SCREEN_WIDTH ) ||
        ( end_col > LCD_ILI9488_SCREEN_WIDTH ) )
        return;

    if( ( start_row > LCD_ILI9488_SCREEN_HEIGHT ) ||
        ( end_row > LCD_ILI9488_SCREEN_HEIGHT ) )
        return;

    if( ( end_col < start_col ) ||
        ( end_row < start_row ) )
        return;

    col_start_stop[0] = start_col >> 8;
    col_start_stop[1] = start_col;
    col_start_stop[2] = (end_col - 1) >> 8;
    col_start_stop[3] = (end_col - 1);
    row_start_stop[0] = start_row >> 8;
    row_start_stop[1] = start_row;
    row_start_stop[2] = (end_row - 1) >> 8;
    row_start_stop[3] = (end_row - 1);

    clr[ 0 ] = colors[color].r; // fill the array with a color (red)
    clr[ 1 ] = colors[color].g; // fill the array with a color (green)
    clr[ 2 ] = colors[color].b; // fill the array with a color (blue)

    lcd_ili9488_more_arg_commands( ctx, LCD_ILI9488_SET_COL_ADDRESS, col_start_stop, 4 );
    lcd_ili9488_more_arg_commands( ctx, LCD_ILI9488_SET_ROW_ADDRESS, row_start_stop, 4 );

    spi_master_select_device( ctx->chip_select );
    digital_out_low( &ctx->dc );
    spi_master_write( &ctx->spi, &cmd, 1 );
    digital_out_high( &ctx->dc );
    
    while( cnt-- )
    {
       spi_master_write( &ctx->spi, &clr[0], 1 );
       spi_master_write( &ctx->spi, &clr[1], 1 ); 
       spi_master_write( &ctx->spi, &clr[2], 1 ); 
    }
    spi_master_deselect_device( ctx->chip_select );  
}

static void draw_area_rgb
( 
    lcd_ili9488_t *ctx, 
    uint16_t start_col, 
    uint16_t start_row, 
    uint16_t end_col,
    uint16_t end_row, 
    const uint8_t *img, 
    uint16_t rgb_bits
)
{
    uint32_t    tmp  = 0;
    uint8_t     cmd  = LCD_ILI9488_WRITE_RAM;
    uint8_t     frb  = 0;
    uint8_t     srb  = 0;
    uint8_t     trb  = 0;
    uint32_t    cnt  = 0;
    
    const uint8_t*  ptr = img + LCD_ILI9488_IMG_HEAD;

    if( ( start_col > LCD_ILI9488_SCREEN_WIDTH ) ||
        ( end_col > LCD_ILI9488_SCREEN_WIDTH ) )
        return;

    if( ( start_row > LCD_ILI9488_SCREEN_HEIGHT ) ||
        ( end_row > LCD_ILI9488_SCREEN_HEIGHT ) )
        return;

    if( ( end_col < start_col ) ||
        ( end_row < start_row ) )
        return;

    col_start_stop[0] = start_col >> 8;
    col_start_stop[1] = start_col;
    col_start_stop[2] = (end_col) >> 8;
    col_start_stop[3] = (end_col);
    row_start_stop[0] = start_row >> 8;
    row_start_stop[1] = start_row;
    row_start_stop[2] = (end_row) >> 8;
    row_start_stop[3] = (end_row);

    lcd_ili9488_more_arg_commands( ctx, LCD_ILI9488_SET_COL_ADDRESS, col_start_stop, 4 );
    lcd_ili9488_more_arg_commands( ctx, LCD_ILI9488_SET_ROW_ADDRESS, row_start_stop, 4 );

    spi_master_select_device( ctx->chip_select );
    digital_out_low( &ctx->dc );
    spi_master_write( &ctx->spi, &cmd, 1 );
    digital_out_high( &ctx->dc );
    
    cnt = (((uint32_t)(img[2] | (img[3]<<8)) * (uint32_t)(img[4] | (img[5]<<8))));

    while( cnt-- )
    {

// NOTE convert 565 color to 888 color, this done because ILI9488 does not do 565 thru SPI interface
//      also for full on 565 color convert to full on 888 ex: red 0xf8 gets coverted to 0xff
        if (rgb_bits == 16)
        {
            frb = ptr[tmp + 1] & 0xf8;
            if (frb == 0xf8) frb = 0xff;
            srb = (ptr[tmp + 1] << 5) | ((ptr[tmp] & 0xe0) >> 3);
            if (srb == 0xfc) srb = 0xff;
            trb = ptr[tmp] << 3;
            if (trb == 0xf8) trb = 0xff;
            spi_master_write( &ctx->spi, &frb, 1 ); // send red color
            spi_master_write( &ctx->spi, &srb, 1 ); // send green color
            spi_master_write( &ctx->spi, &trb, 1 ); // send blue color
            tmp += 2; // remember third color is coverted from 565 3 byte
        }
        if (rgb_bits == 24)
        {
            // NOTE: we send 24 bit data on the (3 bytes) but only 666 get used 
            // color format RRRRRRxx GGGGGGxx BBBBBBxx 262K color
            // this is a limitation of the SPI buss

            frb = ptr[tmp];
            srb = ptr[tmp + 1];
            trb = ptr[tmp + 2];

            spi_master_write( &ctx->spi, &frb, 1 ); // send red color
            spi_master_write( &ctx->spi, &srb, 1 ); // send green color
            spi_master_write( &ctx->spi, &trb, 1 ); // send blue color
            tmp += 3;
        }
    }
    spi_master_deselect_device( ctx->chip_select );  
}

static void pixel( lcd_ili9488_t *ctx, uint16_t col, uint16_t row, uint16_t color )
{
    uint8_t cmd       = LCD_ILI9488_WRITE_RAM;
    uint8_t clr[ 3 ]  = { 0 };

    if( ( col > LCD_ILI9488_SCREEN_WIDTH ) || ( row > LCD_ILI9488_SCREEN_HEIGHT ) )
    {
        return;
    }

    col_start_stop[0] = col >> 8;
    col_start_stop[1] = col;
    col_start_stop[2] = (col - 1) >> 8;
    col_start_stop[3] = (col - 1);
    row_start_stop[0] = row >> 8;
    row_start_stop[1] = row;
    row_start_stop[2] = (row - 1) >> 8;
    row_start_stop[3] = (row - 1);

    clr[ 0 ] = colors[color].r; // fill the array with a color (red)
    clr[ 1 ] = colors[color].g; // fill the array with a color (green)
    clr[ 2 ] = colors[color].b; // fill the array with a color (blue)

    lcd_ili9488_more_arg_commands( ctx, LCD_ILI9488_SET_COL_ADDRESS, col_start_stop, 4 );
    lcd_ili9488_more_arg_commands( ctx, LCD_ILI9488_SET_ROW_ADDRESS, row_start_stop, 4 );

    spi_master_select_device( ctx->chip_select );
    digital_out_low( &ctx->dc );
    spi_master_write( &ctx->spi, &cmd, 1 );
    digital_out_high( &ctx->dc );
    spi_master_write( &ctx->spi, &clr[0], 1 );
    spi_master_write( &ctx->spi, &clr[1], 1 ); 
    spi_master_write( &ctx->spi, &clr[2], 1 ); 
    spi_master_deselect_device( ctx->chip_select ); 
}

static void character( lcd_ili9488_t *ctx, uint16_t ch )
{
    uint8_t     ch_width = 0;
    uint8_t     x_cnt;
    uint8_t     y_cnt;
    uint16_t    x = 0;
    uint16_t    y = 0;
    uint16_t    tmp;
    uint8_t     temp = 0;
    uint8_t     mask = 0;
    uint32_t    offset;
    const uint8_t *ch_table;
    const uint8_t *ch_bitmap;

    if( ch < ctx->font_first_char )
        return;

    if( ch > ctx->font_last_char )
        return;

    offset = 0;
    tmp = (ch - ctx->font_first_char) << 2;
    ch_table = ctx->font_obj + 8 + tmp;
    ch_width = *ch_table;

    offset = (uint32_t)ch_table[1] + ((uint32_t)ch_table[2] << 8) + ((uint32_t)ch_table[3] << 16);

    ch_bitmap = ctx->font_obj + offset;

    y = ctx->y_cord;
    for (y_cnt = 0; y_cnt < ctx->font_height; y_cnt++)
    {
        x = ctx->x_cord;
        mask = 0;
        for( x_cnt = 0; x_cnt < ch_width; x_cnt++ )
        {
            if( !mask )
            {
                temp = *ch_bitmap++;
                mask = 0x01;
            }
            if( temp & mask )
                 pixel( ctx, x, y, ctx->font_color );

            x++;
            mask <<= 1;
        }
        y++;
    }
    ctx->x_cord = x + 1;
}


// ------------------------------------------------------------------------- END

