#ifndef LCD_ILI9488_H
#define LCD_ILI9488_H

/**
 * Any initialization code needed for MCU to function properly.
 * Do not remove this line or clock might not be set correctly.
 */
#ifdef PREINIT_SUPPORTED
#include "preinit.h"
#endif

#ifdef MikroCCoreVersion
    #if MikroCCoreVersion >= 1
        #include "delays.h"
    #endif
#endif

#include "drv_digital_out.h"
#include "drv_digital_in.h"
#include "drv_spi_master.h"
#include "math.h"


// -------------------------------------------------------------- PUBLIC MACROS 
/**
 * \defgroup macros Macros
 * \{
 */

// map the IO for the display
#define LCD_ILI9488_MAP_IO( cfg ) \
   cfg.miso  = PC11; \
   cfg.mosi  = PC12; \
   cfg.sck   = PC10; \
   cfg.cs    = PB4; \
   cfg.rw = 0 ; /* not used */\
   cfg.rst = PD1; \
   cfg.dc = PD2; \
   cfg.en = 0 ; /* not used */
/** \} */

/**
 * \defgroup error_code Error Code
 * \{
 */
#define LCD_ILI9488_RETVAL  uint8_t

#define LCD_ILI9488_OK           0x00
#define LCD_ILI9488_INIT_ERROR   0xFF
/** \} */

/**
 * \define color reference
 * \{
 */
struct RGB_COLOR {
    int8_t r;
    int8_t g;
    int8_t b;
};

// NOTE: if you add more colors please remember to add them in the lcd_ili9488.c file
#define BLACK 0
#define RED 1
#define GREEN 2
#define BLUE 3
#define WHITE 4
#define CYAN 5
#define YELLOW 6
#define DEEP_RED 7

/**
 #define image file we understand (bits per pixel)
 * \{
 */
#define RGB16 16 // RGB 565 16 bits
#define RGB24 24 // RGB 8 8 8 24 bits
#define LCD_ILI9488_IMG_HEAD            0x06
/** \} */

/**
 * \defgroup font_dir Font Direction
 * \{
 */
#define LCD_ILI9488_FO_HORIZONTAL       0x00
#define LCD_ILI9488_FO_VERTICAL         0x01
#define LCD_ILI9488_FO_VERTICAL_COLUMN  0x02
/** \} */


/**
 * \defgroup dev_proper Device Properties
 * \{
 */
#define BYTES_PER_PIXEL 3 // RGB mode is 24bits on spi
#define LANDSCAPE // define landscape mode

#ifndef LANDSCAPE
#define LCD_ILI9488_SCREEN_WIDTH    0x140 // 480
#define LCD_ILI9488_SCREEN_HEIGHT   0x1e0 // 320
#else
#define LCD_ILI9488_SCREEN_WIDTH    0x1e0 // 480
#define LCD_ILI9488_SCREEN_HEIGHT   0x140 // 320
#endif

/** \} */

/**
 * \defgroup commands Commands
 * \{
 */
#define LCD_ILI9488_SET_COL_ADDRESS   0x2A
#define LCD_ILI9488_SET_ROW_ADDRESS   0x2B
#define LCD_ILI9488_WRITE_RAM         0x2C
#define LCD_ILI9488_MEMORY_MODE       0x36
#define LCD_ILI9488_DISPLAY_ON        0x29
#define LCD_ILI9488_DISPLAY_OFF       0x28
#define LCD_ILI9488_SW_RESET          0x01
#define LCD_ILI9488_DISP_INV_CTL      0xB4
#define LCD_ILI9488_DISP_FUN_CTL      0xB6
#define LCD_ILI94880_INTER_PIX_FORMAT 0x3A
#define LCD_ILI9488_SLEEP_OUT         0x11
/** \} */

/** \} */ // End group macro 


// --------------------------------------------------------------- PUBLIC TYPES
/**
 * \defgroup type Types
 * \{
 */

/**
 * @brief Click ctx object definition.
 */
typedef struct
{
    // Output pins 

    digital_out_t rw;
    digital_out_t rst;
    digital_out_t dc;
    digital_out_t en;
    digital_out_t cs;

    // Modules 

    spi_master_t spi;
    pin_name_t chip_select;
    
    const uint8_t*   font_obj;
    uint16_t         font_color;
    uint8_t          font_orientation;
    uint16_t         font_first_char;
    uint16_t         font_last_char;
    uint16_t         font_height;
    uint16_t         x_cord;
    uint16_t         y_cord;

} lcd_ili9488_t;

/**
 * @brief Click configuration structure definition.
 */
typedef struct
{
    // Communication gpio pins 

    pin_name_t miso;
    pin_name_t mosi;
    pin_name_t sck;
    pin_name_t cs;

    // Additional gpio pins 

    pin_name_t rw;
    pin_name_t rst;
    pin_name_t dc;
    pin_name_t en;

    // static variable 

    uint32_t spi_speed;
    spi_master_mode_t spi_mode;
    spi_master_chip_select_polarity_t cs_polarity;

} lcd_ili9488_cfg_t;

/** \} */ // End types group
// ----------------------------------------------- PUBLIC FUNCTION DECLARATIONS

/**
 * \defgroup public_function Public function
 * \{
 */
#ifdef __cplusplus
extern "C"{
#endif

/**
 * @brief Config Object Initialization function.
 *
 * @param cfg  Click configuration structure.
 *
 * @description This function initializes click configuration structure to init state.
 * @note All used pins will be set to unconnected state.
 */
void lcd_ili9488_cfg_setup ( lcd_ili9488_cfg_t *cfg );

/**
 * @brief Initialization function.
 * @param ili9488 Click object.
 * @param cfg Click configuration structure.
 * 
 * @description This function initializes all necessary pins and peripherals used for this click.
 */
LCD_ILI9488_RETVAL lcd_ili9488_init ( lcd_ili9488_t *ctx, lcd_ili9488_cfg_t *cfg );

/**
 * @brief Click Default Configuration function.
 *
 * @param ctx  Click object.
 *
 * @description This function executes default configuration for ILI9488 click.
 */
void lcd_ili9488_default_cfg ( lcd_ili9488_t *ctx );

/**
 * @brief Click Enable/Disable.
 *
 * @param ctx  Context object.
 * @param state  1 - ON / 0 - OFF.
 *
 * @description Function allows user to enable or disable the display.
 */
void lcd_ili9488_enable ( lcd_ili9488_t *ctx, uint8_t state );

/**
 * @brief Click Hardware Reset.
 *
 * @param ctx  Context object.
 *
 * @description Function performs a hardware reset of the click.
 */
void lcd_ili9488_reset( lcd_ili9488_t *ctx );

/**
 * @brief Command Send with more argument.
 *
 * @param ctx  Context object.
 * @param cmd  Command to be sent.
 * @param args  Pointer to command argument.
 * @param args_len  Argument size in bytes.
 *
 * @description Function executes a provided command.
 */
void lcd_ili9488_more_arg_commands ( lcd_ili9488_t *ctx, uint8_t command, uint8_t *args, uint16_t args_len );

/**
 * @brief Command Send with one argument.
 *
 * @param ctx  Context object.
 * @param cmd  Command to be sent.
 * @param args One command argument.
 *
 * @description Function executes a provided command.
 */
void lcd_ili9488_one_arg_commands ( lcd_ili9488_t *ctx, uint8_t command, uint8_t args );

/**
 * @brief Fill Screen.
 *
 * @param ctx  Context object.
 * @param color  RGB color.
 *
 * @description Function fills an entire screen with the provided color.
 */
void lcd_ili9488_fill_screen ( lcd_ili9488_t *ctx, uint16_t color );

/**
 * @brief Draw Rectangle.
 *
 * @param ctx  Context object.
 * @param col_off  Column offset from the left border of the screen.
 * @param row_off  Row offset from the top border of the screen.
 * @param col_end  Column end offset also counted from the left border.
 * @param row_end  Row offset also counted from the top border.
 * @param color  RGB color.
 *
 * @description Function allows user to draw rectangles.
 */
void lcd_ili9488_rectangle( lcd_ili9488_t *ctx, uint16_t col_off, uint16_t row_off, uint16_t col_end, uint16_t row_end, uint16_t color );

/**
 * @brief Draw Line.
 *
 * @param ctx  Context object.
 * @param col_off  Column offset from the left border of the screen.
 * @param row_off  Row offset from the top border of the screen.
 * @param col_end  Column end offset also counted from the left border.
 * @param row_end  Row offset also counted from the top border.
 * @param color  RGB color.
 *
 * @description Function allows user to draw rectangles.
 */
void lcd_ili9488_line( lcd_ili9488_t *ctx, uint16_t col_off, uint16_t row_off, uint16_t col_end, uint16_t row_end, uint16_t color );

/**
 * @brief Draw BMP Image.
 *
 * @param ctx  Context object.
 * @param col_off  Column offset from the left border of the screen.
 * @param row_off  Row offset from the top border of the screen.
 * @param img  Pointer to bmp image.
 *
 * @description Function draws image on the screen. Provided image must be in
 * bmp format with 6 bytes header size. Providing image of any other format may
 * result in unpredictable behaviour.
 */
void lcd_ili9488_image( lcd_ili9488_t *ctx, const uint8_t* img, uint16_t col_off, uint16_t row_off, uint16_t rgb_bits);

/**
 * @brief Draw Text.
 *
 * @param ctx  Context object.
 * @param text  Text string.
 * @param col_off  Column offset from the left border of the screen.
 * @param row_off  Row offset from the top border of the screen.
 *
 * @description Function writes text on the screen.
 */
void lcd_ili9488_text( lcd_ili9488_t *ctx, uint8_t *text, uint16_t x, uint16_t y );

/**
 * @brief Draw circle.
 *
 * @param ctx  Context object.
 * @param x center.
 * @param c center.
 * @param Radius of the circle
 *
 * @description Function writes text on the screen.
 */
void lcd_ili9488_circle ( lcd_ili9488_t *ctx, uint16_t x0, uint16_t y0, uint16_t r, uint16_t color, uint16_t width );

/**
 * @brief Draw filled circle.
 *
 * @param ctx  Context object.
 * @param x center.
 * @param c center.
 * @param Radius of the circle
 *
 * @description Function writes text on the screen.
 */
void lcd_ili9488_fill_circle ( lcd_ili9488_t *ctx, uint16_t x0, uint16_t y0, uint16_t r, uint16_t color );

/**
 * @brief Font Setup.
 *
 * @param ctx  Context object.
 * @param font  Pointer to font definition.
 * @param color  Text color.
 *
 * @description Function setup the parameters needed for @c lcd_ili9488_text function.
 * Before first usage of the @c lcd_ili9488_text it is needed to call this function
 * and setup the all parameters.
 */
void lcd_ili9488_set_font( lcd_ili9488_t *ctx, const uint8_t *font_s, uint16_t color );

#ifdef __cplusplus
}
#endif
#endif  // LCD_ILI9488_H

/** \} */ // End public_function group
/// \}    // End click Driver group  
/*! @} */
// ------------------------------------------------------------------------- END
